;
; BB5A system replacement
;

;
; display 8bits value in hexa (2 chars)
; A=value to translate to text in hexa
; HL=destination buffer (at least 2 bytes)
;
BB5A_PutDigit
push af
ld (BB5A_PutDigitNext+1),a
and #F0
srl a
srl a
srl a
srl a
cp 9
jr c,$+4
add 7
add 48
ld (hl),a
inc hl
BB5A_PutDigitNext:ld a,#12
and #F
cp 9
jr c,$+4
add 7
add 48
ld (hl),a
dec hl
pop af
ret

;
; Setup BB5A system vector replacement
;
; B=video mode
; C=screen width in bytes (may be zero to loop always in the first line!)
;
; will work properly with screens of 8 lines blocks only
;
BB5A_Setup
push af
push bc
push hl
ld h,0
ld l,c
ld (BB5A_ScreenWidth+1),hl
ld a,b
and 3
add a
ld hl,BB5A_SetupModeCallList
add l
ld l,a
ld c,(hl)
inc l
ld b,(hl)
ld (BB5A_CurrentVideoMode+1),bc
pop hl
pop bc
pop af
ret

align 8
BB5A_SetupModeCallList
defw BB5A_MODE0
defw BB5A_MODE1
defw BB5A_MODE2
defw BB5A_MODE0

;
; #BB5A system vector replacement
; use an half width font
;
BB5A
push af
push bc
push de
push hl

cp 13
jr z,BB5A_CarriageReturn
cp 32
jr c,BB5A_END
cp 127
jr nc,BB5A_END

sub 32
add a
ld hl,FonteBB5A
ld d,0
ld e,a
add hl,de
add hl,de ; adresse of char font bits
ex hl,de

BB5A_HL: ld hl,#C000
BB5A_posx: ld c,0
ld b,0

BB5A_CurrentVideoMode: jp BB5A_MODE2

BB5A_Next
ld hl,BB5A_posx+1
inc (hl)
BB5A_END
pop hl
pop de
pop bc
pop af
ret

BB5A_CarriageReturn
ld hl,(BB5A_HL+1)
BB5A_ScreenWidth: ld bc,80
add hl,bc
ld (BB5A_HL+1),hl
xor a
ld (BB5A_posx+1),a
jr BB5A_END

;---- BB5A mode 2 specific display

BB5A_MODE2
srl c
push af
add hl,bc
pop af
ld b,4
jr c,BB5A_MODE2_Odd
; even display
BB5A_MODE2_Even
push bc
ld bc,#800
ld a,(de)
and #F0
ld (hl),a
add hl,bc
ld a,(de)
inc e
add a
add a
add a
add a
ld (hl),a
add hl,bc
pop bc
djnz BB5A_MODE2_Even
jr BB5A_Next

BB5A_MODE2_Odd
push bc
ld bc,#800
ld a,(de)
srl a
srl a
srl a
srl a
or (hl)
ld (hl),a
add hl,bc
ld a,(de)
inc e
and #F
or (hl)
ld (hl),a
add hl,bc
pop bc
djnz BB5A_MODE2_Odd
jr BB5A_Next

;---- BB5A mode 1 specific display

BB5A_MODE1
add hl,bc
ld b,4
BB5A_MODE1_Loop
push bc
ld bc,#800
ld a,(de)
and #F0
ld (hl),a
add hl,bc
ld a,(de)
inc e
add a
add a
add a
add a
ld (hl),a
add hl,bc
pop bc
djnz BB5A_MODE1_Loop
jr BB5A_Next

;---- BB5A mode 0 specific display

BB5A_MODE0
add hl,bc
add hl,bc
ld b,4
BB5A_MODE0_Loop
push bc
ld bc,#800-1
ld a,(de)
and #C0
ld (hl),a
inc hl
ld a,(de)
add a
add a
and #C0
ld (hl),a
add hl,bc
;
ld a,(de)
add a
add a
add a
add a
and #C0
ld (hl),a
inc hl
ld a,(de)
inc e
rrca
rrca
rrca
and #C0
ld (hl),a
add hl,bc

pop bc
djnz BB5A_MODE0_Loop
jp BB5A_Next

align 4
FonteBB5A
db #00,#00,#00,#00
db #04,#44,#40,#40,#0a,#a0,#00,#00
db #06,#f6,#6f,#60,#04,#ec,#e6,#e4
db #0d,#d2,#4b,#b0,#04,#a4,#5a,#d0
db #02,#40,#00,#00,#02,#44,#44,#20
db #04,#22,#22,#40,#00,#a4,#e4,#a0
db #00,#44,#e4,#40,#00,#00,#02,#24
db #00,#00,#e0,#00,#00,#00,#04,#40
db #02,#24,#48,#80
db #0e,#aa,#aa,#e0,#04,#c4,#44,#e0
db #0e,#a2,#e8,#e0,#0e,#a6,#2a,#e0
db #08,#aa,#e2,#20,#0e,#8e,#2a,#e0
db #0e,#8e,#aa,#e0,#0e,#22,#44,#40
db #0e,#ae,#aa,#e0,#0e,#aa,#e2,#e0
db #00,#40,#04,#00,#00,#20,#02,#24
db #00,#24,#84,#20,#00,#0e,#0e,#00
db #00,#84,#24,#80,#0e,#a2,#40,#40
db #06,#bd,#b8,#60,#04,#aa,#ea,#a0
db #0c,#ac,#aa,#c0,#0e,#a8,#8a,#e0
db #0c,#aa,#aa,#c0,#0e,#8c,#88,#e0
db #0e,#8c,#88,#80,#0e,#88,#aa,#e0
db #0a,#ae,#aa,#a0,#0e,#44,#44,#e0
db #02,#22,#aa,#e0,#0a,#ac,#aa,#a0
db #08,#88,#88,#e0,#0a,#ea,#aa,#a0
db #0e,#aa,#aa,#a0,#0e,#aa,#aa,#e0
db #0e,#aa,#e8,#80,#0e,#aa,#ae,#e2
db #0e,#aa,#ca,#a0,#0e,#8e,#22,#e0
db #0e,#44,#44,#40,#0a,#aa,#aa,#e0
db #0a,#aa,#a4,#40,#0a,#aa,#ae,#a0
db #0a,#a4,#4a,#a0,#0a,#aa,#44,#40
db #0e,#24,#48,#e0,#0e,#88,#88,#e0
db #08,#84,#42,#20,#0e,#22,#22,#e0
db #04,#ee,#44,#40,#00,#00,#00,#0f
db #04,#20,#00,#00,#00,#e2,#ea,#e0
db #08,#8e,#aa,#e0,#00,#e8,#88,#e0
db #02,#2e,#aa,#e0,#00,#ea,#e8,#e0
db #06,#8c,#88,#80,#00,#ea,#ae,#2e
db #08,#8e,#aa,#a0,#04,#04,#44,#40
db #02,#02,#22,#ae,#08,#8a,#ca,#a0
db #04,#44,#44,#20,#00,#ae,#aa,#a0
db #00,#ca,#aa,#a0,#00,#ea,#aa,#e0
db #00,#ea,#ae,#88,#00,#ea,#ae,#23
db #00,#68,#88,#80,#00,#e8,#e2,#e0
db #04,#e4,#44,#60,#00,#aa,#aa,#e0
db #00,#aa,#aa,#40,#00,#aa,#ae,#a0
db #00,#aa,#4a,#a0,#00,#aa,#ae,#2e
db #00,#e2,#48,#e0,#06,#48,#44,#60
db #04,#44,#44,#40,#0c,#42,#44,#c0
db #05,#a0,#00,#00,#07,#4e,#44,#f0
